{% extends "stardate/base_stardate.html" %}


{% block title %}{{ post.title }}{% endblock %}

{% block extra_head %}
    <link rel="stylesheet" href="{{ STATIC_URL }}stardate/js/CodeMirror-3.12/lib/codemirror.css">
    <link rel="stylesheet" href="{{ STATIC_URL }}stardate/js/CodeMirror-3.12/theme/monokai.css">
    <link rel="stylesheet" href="{{ STATIC_URL }}stardate/js/highlight.js/styles/monokai.css">
{% endblock %}

{% block page %}
    {% include 'stardate/includes/breadcrumbs.html' %}


    {% block content_title %}
    {% endblock content_title %}


    {% block content %}
        {{form.as_p}}
        <!-- Client Side Rendered HTML Goes Here -->
        <div id="preview">
        </div><!-- End Preview -->

    <script src="//ajax.googleapis.com/ajax/libs/jquery/2.0.0/jquery.min.js"></script>
    <script src="{{ STATIC_URL }}stardate/js/marked-0.2.8/lib/marked.js"></script>
    <script src="{{ STATIC_URL }}stardate/js/CodeMirror-3.12/lib/codemirror.js"></script>
    <script src="{{ STATIC_URL }}stardate/js/CodeMirror-3.12/addon/mode/overlay.js"></script>
    <script src="{{ STATIC_URL }}stardate/js/CodeMirror-3.12/mode/markdown/markdown.js"></script>
    <script src="{{ STATIC_URL }}stardate/js/CodeMirror-3.12/mode/gfm/gfm.js"></script>
    <script src="{{ STATIC_URL }}stardate/js/CodeMirror-3.12/mode/python/python.js"></script>
    <script src="{{ STATIC_URL }}stardate/js/CodeMirror-3.12/mode/javascript/javascript.js"></script>
    <script src="{{ STATIC_URL }}stardate/js/highlight.js/highlight.pack.js"></script>
    <script src="{{ STATIC_URL }}stardate/js/solo.js"></script>
    <script>
    var solo = Solo({
      'markdown': jQuery("#id_body").get(0),
      'preview': jQuery("#preview").get(0),
    });
    </script>
    {% endblock content %}
{% endblock page %}