{% extends "stardate/base_stardate.html" %}

{% block title %}Weblog list{% endblock %}

{% block body_class %}{{ block.super }} blog_list{% endblock %}

{% block content_title %}
    <h1>Weblog list</h1>
{% endblock %}


{% block content %}
    {% if blog_list %}
        <ul>
        {% for blog in blog_list %}
            <li><a href="{{ blog.get_absolute_url }}">{{ blog.name }}</a></li>
        {% endfor %}
        </ul>
    {% else %}
        <p>There are no blogs to display.</p>
    {% endif %}
{% endblock %}
